"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

from pyasoc.configuration import Configuration


class SystemWebHttpODataQuerySelectExpandQueryOption:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "context": "SystemWebHttpODataODataQueryContext",
        "raw_select": "str",
        "raw_expand": "str",
        "validator": "SystemWebHttpODataQueryValidatorsSelectExpandQueryValidator",
        "select_expand_clause": "MicrosoftDataODataQuerySemanticAstSelectExpandClause",
    }

    attribute_map = {
        "context": "Context",
        "raw_select": "RawSelect",
        "raw_expand": "RawExpand",
        "validator": "Validator",
        "select_expand_clause": "SelectExpandClause",
    }

    def __init__(
        self,
        context=None,
        raw_select=None,
        raw_expand=None,
        validator=None,
        select_expand_clause=None,
        _configuration=None,
    ):
        """SystemWebHttpODataQuerySelectExpandQueryOption - a model defined in Swagger"""
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._context = None
        self._raw_select = None
        self._raw_expand = None
        self._validator = None
        self._select_expand_clause = None
        self.discriminator = None

        if context is not None:
            self.context = context
        if raw_select is not None:
            self.raw_select = raw_select
        if raw_expand is not None:
            self.raw_expand = raw_expand
        if validator is not None:
            self.validator = validator
        if select_expand_clause is not None:
            self.select_expand_clause = select_expand_clause

    @property
    def context(self):
        # type: () -> SystemWebHttpODataODataQueryContext
        """Gets the context of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :return: The context of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :rtype: SystemWebHttpODataODataQueryContext
        """
        return self._context

    @context.setter
    def context(self, context):
        """Sets the context of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :param context: The context of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :type: SystemWebHttpODataODataQueryContext
        """

        self._context = context

    @property
    def raw_select(self):
        # type: () -> str
        """Gets the raw_select of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :return: The raw_select of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :rtype: str
        """
        return self._raw_select

    @raw_select.setter
    def raw_select(self, raw_select):
        """Sets the raw_select of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :param raw_select: The raw_select of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :type: str
        """

        self._raw_select = raw_select

    @property
    def raw_expand(self):
        # type: () -> str
        """Gets the raw_expand of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :return: The raw_expand of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :rtype: str
        """
        return self._raw_expand

    @raw_expand.setter
    def raw_expand(self, raw_expand):
        """Sets the raw_expand of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :param raw_expand: The raw_expand of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :type: str
        """

        self._raw_expand = raw_expand

    @property
    def validator(self):
        # type: () -> SystemWebHttpODataQueryValidatorsSelectExpandQueryValidator
        """Gets the validator of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :return: The validator of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :rtype: SystemWebHttpODataQueryValidatorsSelectExpandQueryValidator
        """
        return self._validator

    @validator.setter
    def validator(self, validator):
        """Sets the validator of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :param validator: The validator of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :type: SystemWebHttpODataQueryValidatorsSelectExpandQueryValidator
        """

        self._validator = validator

    @property
    def select_expand_clause(self):
        # type: () -> MicrosoftDataODataQuerySemanticAstSelectExpandClause
        """Gets the select_expand_clause of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :return: The select_expand_clause of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :rtype: MicrosoftDataODataQuerySemanticAstSelectExpandClause
        """
        return self._select_expand_clause

    @select_expand_clause.setter
    def select_expand_clause(self, select_expand_clause):
        """Sets the select_expand_clause of this SystemWebHttpODataQuerySelectExpandQueryOption.


        :param select_expand_clause: The select_expand_clause of this SystemWebHttpODataQuerySelectExpandQueryOption.
        :type: MicrosoftDataODataQuerySemanticAstSelectExpandClause
        """

        self._select_expand_clause = select_expand_clause

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.swagger_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(SystemWebHttpODataQuerySelectExpandQueryOption, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SystemWebHttpODataQuerySelectExpandQueryOption):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SystemWebHttpODataQuerySelectExpandQueryOption):
            return True

        return self.to_dict() != other.to_dict()



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import re  # noqa: F401

# python 2 and python 3 compatibility library
from pyasoc.api_client import ApiClient


class WebhooksApi:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def webhooks_create_association_by_id(self, id, association, **kwargs):
        """Create a new association to the specified Webhook

        A global Webhook is associated to the organization and therefore cannot have other associations.<br />              In order to associate a Webhook to an Application or Assetgroup, you should have access to it
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_create_association_by_id(id, association, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :param AppScanSaaSModelsV2WebhookAssociation association: New association to create (required)
        :return: AppScanSaaSModelsV2WebhookAssociation
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.webhooks_create_association_by_id_with_http_info(id, association, **kwargs)
        else:
            (data) = self.webhooks_create_association_by_id_with_http_info(
                id, association, **kwargs
            )
            return data

    def webhooks_create_association_by_id_with_http_info(self, id, association, **kwargs):
        """Create a new association to the specified Webhook

        A global Webhook is associated to the organization and therefore cannot have other associations.<br />              In order to associate a Webhook to an Application or Assetgroup, you should have access to it
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_create_association_by_id_with_http_info(id, association, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :param AppScanSaaSModelsV2WebhookAssociation association: New association to create (required)
        :return: AppScanSaaSModelsV2WebhookAssociation
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["id", "association"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method webhooks_create_association_by_id" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ("id" not in params or params["id"] is None):
            raise ValueError(
                "Missing the required parameter `id` when calling `webhooks_create_association_by_id`"
            )
        # verify the required parameter 'association' is set
        if self.api_client.client_side_validation and (
            "association" not in params or params["association"] is None
        ):
            raise ValueError(
                "Missing the required parameter `association` when calling `webhooks_create_association_by_id`"
            )

        collection_formats = {}

        path_params = {}
        if "id" in params:
            path_params["id"] = params["id"]

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "association" in params:
            body_params = params["association"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # HTTP header `Content-Type`
        header_params["Content-Type"] = self.api_client.select_header_content_type(
            ["application/json", "text/json", "application/x-www-form-urlencoded"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/v2/Webhooks/{id}/Associations",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="AppScanSaaSModelsV2WebhookAssociation",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def webhooks_create_webhook(self, webhook, **kwargs):
        """Create new Webhook

        In order to create a Webhook for the entire Organization (does not belong to a specific AssetGroup), you should have the capability to access cross asset groups resources<br />              If the Webhook is not Global, you will have to associate it to a resource (AssetGroup or Application) after creation.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_create_webhook(webhook, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AppScanSaaSModelsV2NewWebhook webhook: New Webhook to create (required)
        :return: AppScanSaaSModelsV2Webhook
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.webhooks_create_webhook_with_http_info(webhook, **kwargs)
        else:
            (data) = self.webhooks_create_webhook_with_http_info(webhook, **kwargs)
            return data

    def webhooks_create_webhook_with_http_info(self, webhook, **kwargs):
        """Create new Webhook

        In order to create a Webhook for the entire Organization (does not belong to a specific AssetGroup), you should have the capability to access cross asset groups resources<br />              If the Webhook is not Global, you will have to associate it to a resource (AssetGroup or Application) after creation.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_create_webhook_with_http_info(webhook, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AppScanSaaSModelsV2NewWebhook webhook: New Webhook to create (required)
        :return: AppScanSaaSModelsV2Webhook
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["webhook"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method webhooks_create_webhook" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'webhook' is set
        if self.api_client.client_side_validation and (
            "webhook" not in params or params["webhook"] is None
        ):
            raise ValueError(
                "Missing the required parameter `webhook` when calling `webhooks_create_webhook`"
            )

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "webhook" in params:
            body_params = params["webhook"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # HTTP header `Content-Type`
        header_params["Content-Type"] = self.api_client.select_header_content_type(
            ["application/json", "text/json", "application/x-www-form-urlencoded"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Webhooks",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="AppScanSaaSModelsV2Webhook",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def webhooks_delete_association_by_id(self, id, scope, scope_id, **kwargs):
        """Delete an association from the specified Webhook

        In order to remove association you should have access to Scope object (Application or AssetGroup)
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_delete_association_by_id(id, scope, scope_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :param str scope: Scope of the association to delete (required)
        :param str scope_id: Scope id (AssetGroup or Application Id) (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.webhooks_delete_association_by_id_with_http_info(
                id, scope, scope_id, **kwargs
            )
        else:
            (data) = self.webhooks_delete_association_by_id_with_http_info(
                id, scope, scope_id, **kwargs
            )
            return data

    def webhooks_delete_association_by_id_with_http_info(self, id, scope, scope_id, **kwargs):
        """Delete an association from the specified Webhook

        In order to remove association you should have access to Scope object (Application or AssetGroup)
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_delete_association_by_id_with_http_info(id, scope, scope_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :param str scope: Scope of the association to delete (required)
        :param str scope_id: Scope id (AssetGroup or Application Id) (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["id", "scope", "scope_id"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method webhooks_delete_association_by_id" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ("id" not in params or params["id"] is None):
            raise ValueError(
                "Missing the required parameter `id` when calling `webhooks_delete_association_by_id`"
            )
        # verify the required parameter 'scope' is set
        if self.api_client.client_side_validation and (
            "scope" not in params or params["scope"] is None
        ):
            raise ValueError(
                "Missing the required parameter `scope` when calling `webhooks_delete_association_by_id`"
            )
        # verify the required parameter 'scope_id' is set
        if self.api_client.client_side_validation and (
            "scope_id" not in params or params["scope_id"] is None
        ):
            raise ValueError(
                "Missing the required parameter `scope_id` when calling `webhooks_delete_association_by_id`"
            )

        collection_formats = {}

        path_params = {}
        if "id" in params:
            path_params["id"] = params["id"]

        query_params = []
        if "scope" in params:
            query_params.append(("scope", params["scope"]))
        if "scope_id" in params:
            query_params.append(("scopeId", params["scope_id"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/v2/Webhooks/{id}/Associations",
            "DELETE",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def webhooks_delete_webhook_by_id(self, id, **kwargs):
        """Delete specified Webhook

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_delete_webhook_by_id(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :return: AppScanSaaSModelsV2Webhook
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.webhooks_delete_webhook_by_id_with_http_info(id, **kwargs)
        else:
            (data) = self.webhooks_delete_webhook_by_id_with_http_info(id, **kwargs)
            return data

    def webhooks_delete_webhook_by_id_with_http_info(self, id, **kwargs):
        """Delete specified Webhook

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_delete_webhook_by_id_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :return: AppScanSaaSModelsV2Webhook
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["id"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method webhooks_delete_webhook_by_id" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ("id" not in params or params["id"] is None):
            raise ValueError(
                "Missing the required parameter `id` when calling `webhooks_delete_webhook_by_id`"
            )

        collection_formats = {}

        path_params = {}
        if "id" in params:
            path_params["id"] = params["id"]

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Webhooks/{id}",
            "DELETE",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="AppScanSaaSModelsV2Webhook",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def webhooks_get_associations_by_id(self, id, **kwargs):
        """Get associations of a Webhook

        A global Webhook is associated to the organization and therefore cannot have other associations.</br>OData V3 compliant
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_get_associations_by_id(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :param int top: The max number of records. (Up to 5000)
        :param int skip: The number of records to skip.
        :param str filter: A function that must evaluate to true for a record to be returned. (e.g. Name eq 'sam' or Name eq 'dan'). To filter an Id Property, open the model and see if its type is \"string\" or \"string($uuid)\". - For string no change is needed. - For \"string($uuid)\" add \"Guid\" immediately before the string (e.g. Id eq Guid'00000000-0000-0000-0000-000000000000')
        :param str select: Specifies a subset of properties to return.
        :param str orderby: Determines what values are used to order a collection of records. (e.g. Name,Id)
        :param str expand: Specifies the related resources to be included in line with retrieved resources.
        :return: list[AppScanSaaSModelsV2WebhookAssociation]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.webhooks_get_associations_by_id_with_http_info(id, **kwargs)
        else:
            (data) = self.webhooks_get_associations_by_id_with_http_info(id, **kwargs)
            return data

    def webhooks_get_associations_by_id_with_http_info(self, id, **kwargs):
        """Get associations of a Webhook

        A global Webhook is associated to the organization and therefore cannot have other associations.</br>OData V3 compliant
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_get_associations_by_id_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :param int top: The max number of records. (Up to 5000)
        :param int skip: The number of records to skip.
        :param str filter: A function that must evaluate to true for a record to be returned. (e.g. Name eq 'sam' or Name eq 'dan'). To filter an Id Property, open the model and see if its type is \"string\" or \"string($uuid)\". - For string no change is needed. - For \"string($uuid)\" add \"Guid\" immediately before the string (e.g. Id eq Guid'00000000-0000-0000-0000-000000000000')
        :param str select: Specifies a subset of properties to return.
        :param str orderby: Determines what values are used to order a collection of records. (e.g. Name,Id)
        :param str expand: Specifies the related resources to be included in line with retrieved resources.
        :return: list[AppScanSaaSModelsV2WebhookAssociation]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = [
            "id",
            "top",
            "skip",
            "filter",
            "select",
            "orderby",
            "expand",
        ]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method webhooks_get_associations_by_id" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ("id" not in params or params["id"] is None):
            raise ValueError(
                "Missing the required parameter `id` when calling `webhooks_get_associations_by_id`"
            )

        collection_formats = {}

        path_params = {}
        if "id" in params:
            path_params["id"] = params["id"]

        query_params = []
        if "top" in params:
            query_params.append(("$top", params["top"]))
        if "skip" in params:
            query_params.append(("$skip", params["skip"]))
        if "filter" in params:
            query_params.append(("$filter", params["filter"]))
        if "select" in params:
            query_params.append(("$select", params["select"]))
        if "orderby" in params:
            query_params.append(("$orderby", params["orderby"]))
        if "expand" in params:
            query_params.append(("$expand", params["expand"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/v2/Webhooks/{id}/Associations",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="list[AppScanSaaSModelsV2WebhookAssociation]",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def webhooks_get_webhooks(self, **kwargs):
        """Get all Webhooks

        This action returns all the web hooks that can be accessed usingthe provided access token</br>OData V3 compliant
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_get_webhooks(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int top: The max number of records. (Up to 5000)
        :param int skip: The number of records to skip.
        :param str filter: A function that must evaluate to true for a record to be returned. (e.g. Name eq 'sam' or Name eq 'dan'). To filter an Id Property, open the model and see if its type is \"string\" or \"string($uuid)\". - For string no change is needed. - For \"string($uuid)\" add \"Guid\" immediately before the string (e.g. Id eq Guid'00000000-0000-0000-0000-000000000000')
        :param str select: Specifies a subset of properties to return.
        :param str orderby: Determines what values are used to order a collection of records. (e.g. Name,Id)
        :param str expand: Specifies the related resources to be included in line with retrieved resources.
        :return: list[AppScanSaaSModelsV2Webhook]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.webhooks_get_webhooks_with_http_info(**kwargs)
        else:
            (data) = self.webhooks_get_webhooks_with_http_info(**kwargs)
            return data

    def webhooks_get_webhooks_with_http_info(self, **kwargs):
        """Get all Webhooks

        This action returns all the web hooks that can be accessed usingthe provided access token</br>OData V3 compliant
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_get_webhooks_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int top: The max number of records. (Up to 5000)
        :param int skip: The number of records to skip.
        :param str filter: A function that must evaluate to true for a record to be returned. (e.g. Name eq 'sam' or Name eq 'dan'). To filter an Id Property, open the model and see if its type is \"string\" or \"string($uuid)\". - For string no change is needed. - For \"string($uuid)\" add \"Guid\" immediately before the string (e.g. Id eq Guid'00000000-0000-0000-0000-000000000000')
        :param str select: Specifies a subset of properties to return.
        :param str orderby: Determines what values are used to order a collection of records. (e.g. Name,Id)
        :param str expand: Specifies the related resources to be included in line with retrieved resources.
        :return: list[AppScanSaaSModelsV2Webhook]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = [
            "top",
            "skip",
            "filter",
            "select",
            "orderby",
            "expand",
        ]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method webhooks_get_webhooks" % key
                )
            params[key] = val
        del params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "top" in params:
            query_params.append(("$top", params["top"]))
        if "skip" in params:
            query_params.append(("$skip", params["skip"]))
        if "filter" in params:
            query_params.append(("$filter", params["filter"]))
        if "select" in params:
            query_params.append(("$select", params["select"]))
        if "orderby" in params:
            query_params.append(("$orderby", params["orderby"]))
        if "expand" in params:
            query_params.append(("$expand", params["expand"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Webhooks",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="list[AppScanSaaSModelsV2Webhook]",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def webhooks_update_webhook_by_id(self, id, webhook_model, **kwargs):
        """Update specified Webhook

        If a webhook is set to Global, it will affect all the relevant items that belong to the AssetGroup or to the Organization if the Webhook is defined in the organization level
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_update_webhook_by_id(id, webhook_model, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :param AppScanSaaSModelsV2UpdateWebhook webhook_model: Webhook properties to update (required)
        :return: AppScanSaaSModelsV2Webhook
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.webhooks_update_webhook_by_id_with_http_info(id, webhook_model, **kwargs)
        else:
            (data) = self.webhooks_update_webhook_by_id_with_http_info(id, webhook_model, **kwargs)
            return data

    def webhooks_update_webhook_by_id_with_http_info(self, id, webhook_model, **kwargs):
        """Update specified Webhook

        If a webhook is set to Global, it will affect all the relevant items that belong to the AssetGroup or to the Organization if the Webhook is defined in the organization level
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.webhooks_update_webhook_by_id_with_http_info(id, webhook_model, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: Webhook Id (required)
        :param AppScanSaaSModelsV2UpdateWebhook webhook_model: Webhook properties to update (required)
        :return: AppScanSaaSModelsV2Webhook
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["id", "webhook_model"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method webhooks_update_webhook_by_id" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ("id" not in params or params["id"] is None):
            raise ValueError(
                "Missing the required parameter `id` when calling `webhooks_update_webhook_by_id`"
            )
        # verify the required parameter 'webhook_model' is set
        if self.api_client.client_side_validation and (
            "webhook_model" not in params or params["webhook_model"] is None
        ):
            raise ValueError(
                "Missing the required parameter `webhook_model` when calling `webhooks_update_webhook_by_id`"
            )

        collection_formats = {}

        path_params = {}
        if "id" in params:
            path_params["id"] = params["id"]

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "webhook_model" in params:
            body_params = params["webhook_model"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # HTTP header `Content-Type`
        header_params["Content-Type"] = self.api_client.select_header_content_type(
            ["application/json", "text/json", "application/x-www-form-urlencoded"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Webhooks/{id}",
            "PUT",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="AppScanSaaSModelsV2Webhook",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

from pyasoc.configuration import Configuration


class UserSiteCommonModelsUpdateIastScan:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "config_file_id": "str",
        "name": "str",
        "enable_mail_notifications": "bool",
        "locale": "str",
        "fully_automatic": "bool",
    }

    attribute_map = {
        "config_file_id": "ConfigFileId",
        "name": "Name",
        "enable_mail_notifications": "EnableMailNotifications",
        "locale": "Locale",
        "fully_automatic": "FullyAutomatic",
    }

    def __init__(
        self,
        config_file_id=None,
        name=None,
        enable_mail_notifications=None,
        locale=None,
        fully_automatic=None,
        _configuration=None,
    ):
        """UserSiteCommonModelsUpdateIastScan - a model defined in Swagger"""
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._config_file_id = None
        self._name = None
        self._enable_mail_notifications = None
        self._locale = None
        self._fully_automatic = None
        self.discriminator = None

        if config_file_id is not None:
            self.config_file_id = config_file_id
        if name is not None:
            self.name = name
        if enable_mail_notifications is not None:
            self.enable_mail_notifications = enable_mail_notifications
        if locale is not None:
            self.locale = locale
        if fully_automatic is not None:
            self.fully_automatic = fully_automatic

    @property
    def config_file_id(self):
        # type: () -> str
        """Gets the config_file_id of this UserSiteCommonModelsUpdateIastScan.


        :return: The config_file_id of this UserSiteCommonModelsUpdateIastScan.
        :rtype: str
        """
        return self._config_file_id

    @config_file_id.setter
    def config_file_id(self, config_file_id):
        """Sets the config_file_id of this UserSiteCommonModelsUpdateIastScan.


        :param config_file_id: The config_file_id of this UserSiteCommonModelsUpdateIastScan.
        :type: str
        """

        self._config_file_id = config_file_id

    @property
    def name(self):
        # type: () -> str
        """Gets the name of this UserSiteCommonModelsUpdateIastScan.


        :return: The name of this UserSiteCommonModelsUpdateIastScan.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this UserSiteCommonModelsUpdateIastScan.


        :param name: The name of this UserSiteCommonModelsUpdateIastScan.
        :type: str
        """
        if self._configuration.client_side_validation and name is not None and len(name) > 1024:
            raise ValueError(
                "Invalid value for `name`, length must be less than or equal to `1024`"
            )
        if self._configuration.client_side_validation and name is not None and len(name) < 1:
            raise ValueError(
                "Invalid value for `name`, length must be greater than or equal to `1`"
            )

        self._name = name

    @property
    def enable_mail_notifications(self):
        # type: () -> bool
        """Gets the enable_mail_notifications of this UserSiteCommonModelsUpdateIastScan.


        :return: The enable_mail_notifications of this UserSiteCommonModelsUpdateIastScan.
        :rtype: bool
        """
        return self._enable_mail_notifications

    @enable_mail_notifications.setter
    def enable_mail_notifications(self, enable_mail_notifications):
        """Sets the enable_mail_notifications of this UserSiteCommonModelsUpdateIastScan.


        :param enable_mail_notifications: The enable_mail_notifications of this UserSiteCommonModelsUpdateIastScan.
        :type: bool
        """

        self._enable_mail_notifications = enable_mail_notifications

    @property
    def locale(self):
        # type: () -> str
        """Gets the locale of this UserSiteCommonModelsUpdateIastScan.


        :return: The locale of this UserSiteCommonModelsUpdateIastScan.
        :rtype: str
        """
        return self._locale

    @locale.setter
    def locale(self, locale):
        """Sets the locale of this UserSiteCommonModelsUpdateIastScan.


        :param locale: The locale of this UserSiteCommonModelsUpdateIastScan.
        :type: str
        """
        if self._configuration.client_side_validation and locale is not None and len(locale) > 10:
            raise ValueError(
                "Invalid value for `locale`, length must be less than or equal to `10`"
            )
        if self._configuration.client_side_validation and locale is not None and len(locale) < 0:
            raise ValueError(
                "Invalid value for `locale`, length must be greater than or equal to `0`"
            )

        self._locale = locale

    @property
    def fully_automatic(self):
        # type: () -> bool
        """Gets the fully_automatic of this UserSiteCommonModelsUpdateIastScan.


        :return: The fully_automatic of this UserSiteCommonModelsUpdateIastScan.
        :rtype: bool
        """
        return self._fully_automatic

    @fully_automatic.setter
    def fully_automatic(self, fully_automatic):
        """Sets the fully_automatic of this UserSiteCommonModelsUpdateIastScan.


        :param fully_automatic: The fully_automatic of this UserSiteCommonModelsUpdateIastScan.
        :type: bool
        """

        self._fully_automatic = fully_automatic

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.swagger_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(UserSiteCommonModelsUpdateIastScan, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UserSiteCommonModelsUpdateIastScan):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, UserSiteCommonModelsUpdateIastScan):
            return True

        return self.to_dict() != other.to_dict()



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

from pyasoc.configuration import Configuration


class UtilitiesStructuresRegulationReportOptions:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "regulation_report_type": "str",
        "report_file_type": "str",
        "title": "str",
        "notes": "str",
        "locale": "str",
    }

    attribute_map = {
        "regulation_report_type": "RegulationReportType",
        "report_file_type": "ReportFileType",
        "title": "Title",
        "notes": "Notes",
        "locale": "Locale",
    }

    def __init__(
        self,
        regulation_report_type=None,
        report_file_type=None,
        title=None,
        notes=None,
        locale=None,
        _configuration=None,
    ):
        """UtilitiesStructuresRegulationReportOptions - a model defined in Swagger"""
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._regulation_report_type = None
        self._report_file_type = None
        self._title = None
        self._notes = None
        self._locale = None
        self.discriminator = None

        self.regulation_report_type = regulation_report_type
        if report_file_type is not None:
            self.report_file_type = report_file_type
        if title is not None:
            self.title = title
        if notes is not None:
            self.notes = notes
        if locale is not None:
            self.locale = locale

    @property
    def regulation_report_type(self):
        # type: () -> str
        """Gets the regulation_report_type of this UtilitiesStructuresRegulationReportOptions.


        :return: The regulation_report_type of this UtilitiesStructuresRegulationReportOptions.
        :rtype: str
        """
        return self._regulation_report_type

    @regulation_report_type.setter
    def regulation_report_type(self, regulation_report_type):
        """Sets the regulation_report_type of this UtilitiesStructuresRegulationReportOptions.


        :param regulation_report_type: The regulation_report_type of this UtilitiesStructuresRegulationReportOptions.
        :type: str
        """
        if self._configuration.client_side_validation and regulation_report_type is None:
            raise ValueError("Invalid value for `regulation_report_type`, must not be `None`")
        allowed_values = [
            "None",
            "OwaspTop10_2017",
            "Sans25",
            "EuGdpr_2016",
            "PCI",
            "Hipaa",
            "OwaspTop10Mobile_2016",
            "ISO27001",
            "ISO27002",
            "Wasc",
            "Nist",
            "Sox",
            "Fisma",
            "Fippa",
            "Efta",
            "DisaStig",
            "Padss",
            "OwaspTop10_2021",
            "OwaspTop10OpenApi_2019",
            "Ccpa",
        ]
        if (
            self._configuration.client_side_validation
            and regulation_report_type not in allowed_values
        ):
            raise ValueError(
                "Invalid value for `regulation_report_type` ({}), must be one of {}".format(
                    regulation_report_type, allowed_values
                )
            )

        self._regulation_report_type = regulation_report_type

    @property
    def report_file_type(self):
        # type: () -> str
        """Gets the report_file_type of this UtilitiesStructuresRegulationReportOptions.


        :return: The report_file_type of this UtilitiesStructuresRegulationReportOptions.
        :rtype: str
        """
        return self._report_file_type

    @report_file_type.setter
    def report_file_type(self, report_file_type):
        """Sets the report_file_type of this UtilitiesStructuresRegulationReportOptions.


        :param report_file_type: The report_file_type of this UtilitiesStructuresRegulationReportOptions.
        :type: str
        """
        allowed_values = ["Pdf", "Html", "Xml", "Csv", "Sarif"]
        if self._configuration.client_side_validation and report_file_type not in allowed_values:
            raise ValueError(
                "Invalid value for `report_file_type` ({}), must be one of {}".format(
                    report_file_type, allowed_values
                )
            )

        self._report_file_type = report_file_type

    @property
    def title(self):
        # type: () -> str
        """Gets the title of this UtilitiesStructuresRegulationReportOptions.


        :return: The title of this UtilitiesStructuresRegulationReportOptions.
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this UtilitiesStructuresRegulationReportOptions.


        :param title: The title of this UtilitiesStructuresRegulationReportOptions.
        :type: str
        """

        self._title = title

    @property
    def notes(self):
        # type: () -> str
        """Gets the notes of this UtilitiesStructuresRegulationReportOptions.


        :return: The notes of this UtilitiesStructuresRegulationReportOptions.
        :rtype: str
        """
        return self._notes

    @notes.setter
    def notes(self, notes):
        """Sets the notes of this UtilitiesStructuresRegulationReportOptions.


        :param notes: The notes of this UtilitiesStructuresRegulationReportOptions.
        :type: str
        """

        self._notes = notes

    @property
    def locale(self):
        # type: () -> str
        """Gets the locale of this UtilitiesStructuresRegulationReportOptions.


        :return: The locale of this UtilitiesStructuresRegulationReportOptions.
        :rtype: str
        """
        return self._locale

    @locale.setter
    def locale(self, locale):
        """Sets the locale of this UtilitiesStructuresRegulationReportOptions.


        :param locale: The locale of this UtilitiesStructuresRegulationReportOptions.
        :type: str
        """

        self._locale = locale

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.swagger_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(UtilitiesStructuresRegulationReportOptions, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UtilitiesStructuresRegulationReportOptions):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, UtilitiesStructuresRegulationReportOptions):
            return True

        return self.to_dict() != other.to_dict()



import enum
import json
import os
import subprocess  # nosec: B404
import sys
import time
from contextlib import contextmanager
from datetime import datetime, timedelta
from os import PathLike
from pathlib import Path
from typing import Iterable, TextIO

from pytz import utc

from .api.account_api import AccountApi
from .api.apps_api import AppsApi
from .api.asset_groups_api import AssetGroupsApi
from .api.file_upload_api import FileUploadApi
from .api.issues_api import IssuesApi
from .api.scans_api import ScansApi
from .api_client import ApiClient
from .configuration import Configuration
from .models import (
    AppScanSaaSModelsV2MinScanData,
    AppScanSaaSModelsV2ScanExecution,
    AppScanSaaSModelsV2StaticAnalyzerScan,
    CommonModelsASMModelsApplicationCreateModel,
    CommonModelsASMModelsApplicationModel,
    SystemWebHttpODataPageResultUtilitiesIssueModel,
    UserSiteCommonModelsApiKey,
    UserSiteCommonModelsNewStaticAnalyzerScan,
    UserSiteCommonModelsScanExecute,
    UtilitiesIssueModel,
    UtilitiesUpdateIssue,
)


@contextmanager
def cd(path: str | PathLike):
    old_dir = os.getcwd()
    os.chdir(path)
    try:
        yield
    finally:
        os.chdir(old_dir)


def parse_issue(issue):
    ret = {
        "External Id": issue["id"],
        # "Issue Type": ,
        "Location": f'{issue["location"]["file"]} ({issue["location"]["start_line"]})',
        "Severity": issue["severity"],
        # "Context": ,
        "CVE": issue["cve"],
        # "CVSS": ,
        # "Domain": ,
        # "Element": ,
        # "Host": ,
        "Line": issue["location"]["start_line"],
        # "Path": ,
        # "Port": ,
        "File Name": issue["location"]["file"],
    }
    if ret["Severity"] == "Info":
        ret["Severity"] = "Informational"
    desc = issue.get("description", "").strip().replace(",", "")
    msg = issue.get("message", "").strip().replace(",", "")
    if desc and msg:
        ret["Issue Type"] = msg
        ret["Context"] = desc
    elif desc:
        ret["Issue Type"] = desc
    elif msg:
        ret["Issue Type"] = msg
    ret["Issue Type"] = f'{issue["scanner"]["name"]}: {ret["Issue Type"]}'
    return ret


RISK_RATING_COLORS = {
    "Unknown": "blue",
    "Low": "brightgreen",
    "Medium": "yellow",
    "High": "orange",
    "Critical": "red",
}


class IssueLevels(str, enum.Enum):
    CRITICAL = "critical"
    HIGH = "high"
    MEDIUM = "medium"
    LOW = "low"
    INFORMATIONAL = "informational"


class Asoc:
    def __init__(
        self,
        key_id,
        key_secret,
        app_id: str | None = None,
        app_name: str | None = None,
        create_new: bool = True,
        asset_group_id: str | None = None,
        scan_name="main",
        business_impact: str = "Low",
    ):
        self.scan_id = self.exec_id = None
        self.key_id = key_id
        self.key_secret = key_secret
        self._token_expires = datetime(1990, 1, 1, 1, 1).astimezone(utc)
        self._config = Configuration()
        self._config.api_key_prefix = {"Authorization": "Bearer"}
        if proxy := os.getenv("https_proxy", os.getenv("HTTPS_PROXY")):
            self._config.proxy = proxy
        self._client = ApiClient(configuration=self._config)
        self._config.refresh_api_key_hook = self._get_token

        self._account = AccountApi(api_client=self._client)
        self._issues = IssuesApi(api_client=self._client)
        self._scans = ScansApi(api_client=self._client)
        self._apps = AppsApi(api_client=self._client)
        self._asset_group = asset_group_id

        self.scan_name = scan_name
        if app_id is None and app_name is None:
            raise ValueError("app_id or app_name must be specified")
        if app_id is not None and app_name is not None:
            raise ValueError("app_id and app_name cannot be specified at the same time")
        if app_name is not None:
            app_items = self._apps.apps_get_apps_page(filter=f"Name eq '{app_name}'").items
            if len(app_items) > 1:
                raise LookupError(f"Found more than one app with name {app_name}?")
            if len(app_items) == 0:
                if not create_new:
                    raise ValueError(f"no app with name {app_name} found, and not creating new")
                app_created = self._apps.apps_create_app(
                    model=CommonModelsASMModelsApplicationCreateModel(
                        name=app_name,
                        asset_group_id=self.asset_group,
                        business_impact=business_impact,
                    )
                )
                self.app_id = app_created.id
            else:
                self.app_id = app_items[0].id
        else:
            self.app_id = app_id

    @property
    def asset_group(self):
        if self._asset_group is None:
            aga = AssetGroupsApi(self._client)
            self._asset_group = [a for a in aga.asset_groups_get_asset_groups() if a.is_default][
                0
            ].id
        return self._asset_group

    def _get_token(self, obj: Configuration):
        if self._token_expires > datetime.now().astimezone(utc):
            print("reusing token")
            return

        r = self._account.account_api_key_login(
            UserSiteCommonModelsApiKey(
                key_id=self.key_id,
                key_secret=self.key_secret,
            ),
            # _request_timeout=10,
        )
        obj.api_key["Authorization"] = r.token
        self._token_expires = r.expire

    def _new_scan_start(self, file_id, **kwargs) -> AppScanSaaSModelsV2StaticAnalyzerScan:
        return self._scans.scans_create_static_analyzer_scan(
            UserSiteCommonModelsNewStaticAnalyzerScan(
                app_id=self.app_id,
                application_file_id=file_id,
                scan_name=self.scan_name,
                enable_mail_notification=False,
                comment=kwargs.get("comment", ""),
                personal=kwargs.get("personal", False),
            )
        )

    def _old_scan_start(self, scan_id: str, file_id: str) -> AppScanSaaSModelsV2ScanExecution:
        return self._scans.scans_execute_scan_by_scanid(
            scan_id=scan_id, model=UserSiteCommonModelsScanExecute(file_id=file_id)
        )

    # region external issue handling
    def get_external_issues(self) -> list[UtilitiesIssueModel]:
        r: SystemWebHttpODataPageResultUtilitiesIssueModel
        r = self._issues.issues_get_issues_for_scope_by_scope_and_scopeid(
            scope="Application",
            scope_id=self.app_id,
            top=5000,
            filter="length(ExternalId) gt 1",
            # _request_timeout=10,
        )
        return r.items

    def filter_issues(self, files: Iterable[str | PathLike]):
        issues = {}
        for f in files:
            try:
                with open(f, encoding="utf8") as fh:
                    data = json.load(fh)
                    for v in data.get("vulnerabilities", []):
                        iss = parse_issue(v)
                        if iss["External Id"] not in issues:
                            issues[iss["External Id"]] = iss
                        # break
            except (json.JSONDecodeError, KeyError):
                pass
        existing_issues = self.get_external_issues()
        existing_open: dict[str, UtilitiesIssueModel] = {
            a.external_id: a
            for a in existing_issues
            if a.status
            in (
                "Open",
                "Reopened",
                "New",
            )
        }
        existing_fixed: dict[str, UtilitiesIssueModel] = {
            a.external_id: a for a in existing_issues if a.status in ("Fixed",)
        }
        existing_passed: dict[str, UtilitiesIssueModel] = {
            a.external_id: a
            for a in existing_issues
            if a.status
            in (
                "Passed",
                "Noise",
            )
        }

        new_issues: list[dict] = []
        to_fix: list[UtilitiesIssueModel] = []
        to_reopen: list[UtilitiesIssueModel] = []
        stay_open: list[UtilitiesIssueModel] = []
        for eid, v in issues.items():
            if eid in existing_open:
                # is already open
                stay_open.append(existing_open.pop(eid))
            elif eid in existing_fixed:
                # need to reopen
                to_reopen.append(existing_fixed.pop(eid))
            elif eid in existing_passed:
                # has been changed to passed or fixed, ignore
                pass
            else:
                new_issues.append(v)
        for v in existing_open.values():
            to_fix.append(v)
        return new_issues, to_fix, to_reopen, stay_open

    def change_issue_status(self, issue_id: str, status):
        self._issues.issues_update_issue_by_id(
            id=issue_id, issue_data=UtilitiesUpdateIssue(status=status)
        )

    def change_issue_status_bulk(self, issue_ids: list[str], status):
        all_ids = ":".join(issue_ids)
        return self._issues.issues_update_issues_for_scope_by_scope_and_scopeid(
            scope="Application",
            scope_id=self.app_id,
            issue_data=UtilitiesUpdateIssue(status=status),
            odata_filter=f"substringof(Id,'{all_ids}')",
        )
        # for ii in issue_ids:
        #     self.change_issue_status(ii, status)

    def import_file(self, file: str | bytes | PathLike):
        return self._issues.issues_import_file(
            file_to_upload=file, app_id=self.app_id, scan_name=self.scan_name
        )

    # endregion

    def upload_file(self, file: str | bytes | PathLike) -> str:
        fa = FileUploadApi(self._client)
        return fa.file_upload_upload(file_to_upload=file).file_id

    def get_app(self) -> CommonModelsASMModelsApplicationModel:
        aa = AppsApi(self._client)
        return aa.apps_get_app_by_id(self.app_id)

    def write_status_badge(self, handle: TextIO):
        app_status = self.get_app()
        self._write_badge(
            handle,
            label="AppScan Risk Rating",
            message=app_status.risk_rating,
            color=RISK_RATING_COLORS[app_status.risk_rating],
        )

    def write_num_issues_badge(
        self,
        handle: TextIO,
        cutoff_level: IssueLevels | str = IssueLevels.MEDIUM,
        color_cutoffs: tuple[int, int, int] = (5, 10, 25),
    ):
        app = self.get_app()
        if len(set(color_cutoffs)) != 3 or tuple(sorted(color_cutoffs)) != color_cutoffs:
            raise ValueError(f"invalid cutoffs {color_cutoffs}")
        if not isinstance(cutoff_level, IssueLevels):
            for il in IssueLevels:
                if il.value.startswith(str(cutoff_level).lower()):
                    cutoff_level = il
                    break
            else:
                raise ValueError(f"cutoff not in {[a.value for a in IssueLevels]}")
        total = 0
        for il in IssueLevels:
            total += getattr(app, il.value + "_issues")
            if il == cutoff_level:
                break
        color = "brightgreen"
        for co, c_opt in zip(color_cutoffs, ("yellow", "orange", "red")):
            if total >= co:
                color = c_opt
            else:
                break
        self._write_badge(
            handle,
            label="AppScan Issues",
            message=f"{total}",
            color=color,
        )

    @staticmethod
    def _write_badge(handle: TextIO, label: str, message: str, color: str = "brightgreen"):
        try:
            # noinspection PyPackageRequirements
            from javascript import require
        except ImportError as e:
            raise ImportError(
                "javascript not installed, install with 'pip install pyasoc[badges]'"
            ) from e

        bm = require("badge-maker")
        handle.write(
            bm.makeBadge(
                {
                    "label": label,
                    "message": message,
                    "color": color,
                    "style": "flat",
                }
            )
        )

    def scan_exists(self) -> bool | AppScanSaaSModelsV2MinScanData:
        q = self._scans.scans_get_scans_page2(
            filter=f"AppId eq Guid'{self.app_id}' and Name eq '{self.scan_name}'"
        )
        if len(q.items) == 0:
            return False
        return q.items[0]

    def get_scan(self, execution_id: str | None = None) -> AppScanSaaSModelsV2ScanExecution:
        if execution_id is None:
            if self.exec_id is None:
                execution_id = self.scan_exists().latest_execution.id
            else:
                execution_id = self.exec_id
        return self._scans.scans_execution_by_executionid(execution_id=execution_id)

    def package(
        self,
        working_dir: str | bytes | PathLike = ".",
        extra_options: None | list[str] = None,
        packager: str | bytes | PathLike = "appscan.sh",
    ):
        if extra_options is None:
            extra_options = []
        with cd(working_dir):
            subprocess.run(  # nosec: B603
                [packager, "prepare", "-n", self.scan_name + ".irx", *extra_options],
                check=True,
                stderr=sys.stderr,
                stdout=sys.stdout,
            )

    def create_scan(self, working_dir: str | bytes | PathLike = ".", **kwargs):
        with cd(working_dir):
            if not Path(self.scan_name + ".irx").exists():
                self.package(
                    working_dir=working_dir,
                    extra_options=kwargs.get("extra_options"),
                    packager=kwargs.get("packager", "appscan.sh"),
                )
            file_id = self.upload_file(self.scan_name + ".irx")
            if scan_info := self.scan_exists():
                scan_id = scan_info.id
                exec_info = self._old_scan_start(scan_id=scan_id, file_id=file_id)
                exec_id = exec_info.id
            else:
                exec_info = self._new_scan_start(file_id=file_id, **kwargs)
                exec_id = exec_info.latest_execution.id
                scan_id = exec_info.latest_execution.scan_id
        self.scan_id = scan_id
        self.exec_id = exec_id
        print(exec_id)
        return scan_id, exec_id

    def wait(self, interval: int = 10, timeout_minutes: int = 60):
        start = datetime.now()
        while True:
            scan_info = self.get_scan()
            print(scan_info.id)
            if scan_info.status in ("Ready", "Failed"):
                break
            if datetime.now() - start > timedelta(minutes=timeout_minutes):
                raise TimeoutError("Scan timed out")
            time.sleep(interval)
        return scan_info



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import re  # noqa: F401

# python 2 and python 3 compatibility library
from pyasoc.api_client import ApiClient


class DomainsApi:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def domains_delete_domain_by_id(self, id, **kwargs):
        """Delete domain

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_delete_domain_by_id(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: domain id (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.domains_delete_domain_by_id_with_http_info(id, **kwargs)
        else:
            (data) = self.domains_delete_domain_by_id_with_http_info(id, **kwargs)
            return data

    def domains_delete_domain_by_id_with_http_info(self, id, **kwargs):
        """Delete domain

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_delete_domain_by_id_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: domain id (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["id"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method domains_delete_domain_by_id" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ("id" not in params or params["id"] is None):
            raise ValueError(
                "Missing the required parameter `id` when calling `domains_delete_domain_by_id`"
            )

        collection_formats = {}

        path_params = {}
        if "id" in params:
            path_params["id"] = params["id"]

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Domains/{id}",
            "DELETE",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def domains_download_file_by_id(self, id, **kwargs):
        """Download domain verification HTML

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_download_file_by_id(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: domain id (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.domains_download_file_by_id_with_http_info(id, **kwargs)
        else:
            (data) = self.domains_download_file_by_id_with_http_info(id, **kwargs)
            return data

    def domains_download_file_by_id_with_http_info(self, id, **kwargs):
        """Download domain verification HTML

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_download_file_by_id_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: domain id (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["id"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method domains_download_file_by_id" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ("id" not in params or params["id"] is None):
            raise ValueError(
                "Missing the required parameter `id` when calling `domains_download_file_by_id`"
            )

        collection_formats = {}

        path_params = {}
        if "id" in params:
            path_params["id"] = params["id"]

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Domains/DownloadFile/{id}",
            "POST",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def domains_get_domains_as_page(self, **kwargs):
        """Get domains

        OData V3 compliant
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_get_domains_as_page(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int top: The max number of records. (Up to 5000)
        :param int skip: The number of records to skip.
        :param str filter: A function that must evaluate to true for a record to be returned. (e.g. Name eq 'sam' or Name eq 'dan'). To filter an Id Property, open the model and see if its type is \"string\" or \"string($uuid)\". - For string no change is needed. - For \"string($uuid)\" add \"Guid\" immediately before the string (e.g. Id eq Guid'00000000-0000-0000-0000-000000000000')
        :param str select: Specifies a subset of properties to return.
        :param str orderby: Determines what values are used to order a collection of records. (e.g. Name,Id)
        :param str expand: Specifies the related resources to be included in line with retrieved resources.
        :param str inlinecount: Specifies that the response to the request MUST include the count of the number of entities in the collection of entities.
        :return: SystemWebHttpODataPageResultCommonModelsDomainModel
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.domains_get_domains_as_page_with_http_info(**kwargs)
        else:
            (data) = self.domains_get_domains_as_page_with_http_info(**kwargs)
            return data

    def domains_get_domains_as_page_with_http_info(self, **kwargs):
        """Get domains

        OData V3 compliant
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_get_domains_as_page_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int top: The max number of records. (Up to 5000)
        :param int skip: The number of records to skip.
        :param str filter: A function that must evaluate to true for a record to be returned. (e.g. Name eq 'sam' or Name eq 'dan'). To filter an Id Property, open the model and see if its type is \"string\" or \"string($uuid)\". - For string no change is needed. - For \"string($uuid)\" add \"Guid\" immediately before the string (e.g. Id eq Guid'00000000-0000-0000-0000-000000000000')
        :param str select: Specifies a subset of properties to return.
        :param str orderby: Determines what values are used to order a collection of records. (e.g. Name,Id)
        :param str expand: Specifies the related resources to be included in line with retrieved resources.
        :param str inlinecount: Specifies that the response to the request MUST include the count of the number of entities in the collection of entities.
        :return: SystemWebHttpODataPageResultCommonModelsDomainModel
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = [
            "top",
            "skip",
            "filter",
            "select",
            "orderby",
            "expand",
            "inlinecount",
        ]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method domains_get_domains_as_page" % key
                )
            params[key] = val
        del params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "top" in params:
            query_params.append(("$top", params["top"]))
        if "skip" in params:
            query_params.append(("$skip", params["skip"]))
        if "filter" in params:
            query_params.append(("$filter", params["filter"]))
        if "select" in params:
            query_params.append(("$select", params["select"]))
        if "orderby" in params:
            query_params.append(("$orderby", params["orderby"]))
        if "expand" in params:
            query_params.append(("$expand", params["expand"]))
        if "inlinecount" in params:
            query_params.append(("$inlinecount", params["inlinecount"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Domains",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="SystemWebHttpODataPageResultCommonModelsDomainModel",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def domains_resend_mail_by_id(self, id, mail_prefix, **kwargs):
        """Resend domain verification mail

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_resend_mail_by_id(id, mail_prefix, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: domain id (required)
        :param str mail_prefix: mail prefix (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.domains_resend_mail_by_id_with_http_info(id, mail_prefix, **kwargs)
        else:
            (data) = self.domains_resend_mail_by_id_with_http_info(id, mail_prefix, **kwargs)
            return data

    def domains_resend_mail_by_id_with_http_info(self, id, mail_prefix, **kwargs):
        """Resend domain verification mail

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_resend_mail_by_id_with_http_info(id, mail_prefix, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: domain id (required)
        :param str mail_prefix: mail prefix (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["id", "mail_prefix"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method domains_resend_mail_by_id" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ("id" not in params or params["id"] is None):
            raise ValueError(
                "Missing the required parameter `id` when calling `domains_resend_mail_by_id`"
            )
        # verify the required parameter 'mail_prefix' is set
        if self.api_client.client_side_validation and (
            "mail_prefix" not in params or params["mail_prefix"] is None
        ):
            raise ValueError(
                "Missing the required parameter `mail_prefix` when calling `domains_resend_mail_by_id`"
            )

        collection_formats = {}

        path_params = {}
        if "id" in params:
            path_params["id"] = params["id"]

        query_params = []
        if "mail_prefix" in params:
            query_params.append(("mailPrefix", params["mail_prefix"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Domains/ResendMail/{id}",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="object",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def domains_update_domain_by_id(self, id, update_model, **kwargs):
        """Update domain

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_update_domain_by_id(id, update_model, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: Domain Id (required)
        :param CommonModelsUpdateDomainModel update_model:  (required)
        :return: CommonModelsDomainModel
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.domains_update_domain_by_id_with_http_info(id, update_model, **kwargs)
        else:
            (data) = self.domains_update_domain_by_id_with_http_info(id, update_model, **kwargs)
            return data

    def domains_update_domain_by_id_with_http_info(self, id, update_model, **kwargs):
        """Update domain

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.domains_update_domain_by_id_with_http_info(id, update_model, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param int id: Domain Id (required)
        :param CommonModelsUpdateDomainModel update_model:  (required)
        :return: CommonModelsDomainModel
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["id", "update_model"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method domains_update_domain_by_id" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'id' is set
        if self.api_client.client_side_validation and ("id" not in params or params["id"] is None):
            raise ValueError(
                "Missing the required parameter `id` when calling `domains_update_domain_by_id`"
            )
        # verify the required parameter 'update_model' is set
        if self.api_client.client_side_validation and (
            "update_model" not in params or params["update_model"] is None
        ):
            raise ValueError(
                "Missing the required parameter `update_model` when calling `domains_update_domain_by_id`"
            )

        collection_formats = {}

        path_params = {}
        if "id" in params:
            path_params["id"] = params["id"]

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if "update_model" in params:
            body_params = params["update_model"]
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(
            ["application/json", "text/json", "text/csv"]
        )

        # HTTP header `Content-Type`
        header_params["Content-Type"] = self.api_client.select_header_content_type(
            ["application/json", "text/json", "application/x-www-form-urlencoded"]
        )

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Domains/{id}",
            "PUT",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="CommonModelsDomainModel",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

from pyasoc.configuration import Configuration


class CommonModelsSCXAccountSCXSubscription:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "license_key_postfix": "str",
        "subscription_id": "int",
        "overage_support": "bool",
        "expiration_date": "datetime",
        "renewal_date": "datetime",
        "purchase_date": "datetime",
        "offering_type": "str",
        "n_seats": "int",
        "n_taken_seats": "int",
        "n_executions": "int",
        "n_apps": "int",
        "full_report": "bool",
        "max_concurrent_scans": "int",
        "purchase_supplier": "str",
        "state": "str",
        "include_open_source": "bool",
        "include_iast": "bool",
        "max_iast_concurrency": "int",
        "is3_pm_order": "bool",
    }

    attribute_map = {
        "license_key_postfix": "LicenseKeyPostfix",
        "subscription_id": "SubscriptionId",
        "overage_support": "OverageSupport",
        "expiration_date": "ExpirationDate",
        "renewal_date": "RenewalDate",
        "purchase_date": "PurchaseDate",
        "offering_type": "OfferingType",
        "n_seats": "NSeats",
        "n_taken_seats": "NTakenSeats",
        "n_executions": "NExecutions",
        "n_apps": "NApps",
        "full_report": "FullReport",
        "max_concurrent_scans": "MaxConcurrentScans",
        "purchase_supplier": "PurchaseSupplier",
        "state": "State",
        "include_open_source": "IncludeOpenSource",
        "include_iast": "IncludeIAST",
        "max_iast_concurrency": "MaxIASTConcurrency",
        "is3_pm_order": "Is3PMOrder",
    }

    def __init__(
        self,
        license_key_postfix=None,
        subscription_id=None,
        overage_support=None,
        expiration_date=None,
        renewal_date=None,
        purchase_date=None,
        offering_type=None,
        n_seats=None,
        n_taken_seats=None,
        n_executions=None,
        n_apps=None,
        full_report=None,
        max_concurrent_scans=None,
        purchase_supplier=None,
        state=None,
        include_open_source=None,
        include_iast=None,
        max_iast_concurrency=None,
        is3_pm_order=None,
        _configuration=None,
    ):
        """CommonModelsSCXAccountSCXSubscription - a model defined in Swagger"""
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._license_key_postfix = None
        self._subscription_id = None
        self._overage_support = None
        self._expiration_date = None
        self._renewal_date = None
        self._purchase_date = None
        self._offering_type = None
        self._n_seats = None
        self._n_taken_seats = None
        self._n_executions = None
        self._n_apps = None
        self._full_report = None
        self._max_concurrent_scans = None
        self._purchase_supplier = None
        self._state = None
        self._include_open_source = None
        self._include_iast = None
        self._max_iast_concurrency = None
        self._is3_pm_order = None
        self.discriminator = None

        if license_key_postfix is not None:
            self.license_key_postfix = license_key_postfix
        if subscription_id is not None:
            self.subscription_id = subscription_id
        if overage_support is not None:
            self.overage_support = overage_support
        if expiration_date is not None:
            self.expiration_date = expiration_date
        if renewal_date is not None:
            self.renewal_date = renewal_date
        if purchase_date is not None:
            self.purchase_date = purchase_date
        if offering_type is not None:
            self.offering_type = offering_type
        if n_seats is not None:
            self.n_seats = n_seats
        if n_taken_seats is not None:
            self.n_taken_seats = n_taken_seats
        if n_executions is not None:
            self.n_executions = n_executions
        if n_apps is not None:
            self.n_apps = n_apps
        if full_report is not None:
            self.full_report = full_report
        if max_concurrent_scans is not None:
            self.max_concurrent_scans = max_concurrent_scans
        if purchase_supplier is not None:
            self.purchase_supplier = purchase_supplier
        if state is not None:
            self.state = state
        if include_open_source is not None:
            self.include_open_source = include_open_source
        if include_iast is not None:
            self.include_iast = include_iast
        if max_iast_concurrency is not None:
            self.max_iast_concurrency = max_iast_concurrency
        if is3_pm_order is not None:
            self.is3_pm_order = is3_pm_order

    @property
    def license_key_postfix(self):
        # type: () -> str
        """Gets the license_key_postfix of this CommonModelsSCXAccountSCXSubscription.


        :return: The license_key_postfix of this CommonModelsSCXAccountSCXSubscription.
        :rtype: str
        """
        return self._license_key_postfix

    @license_key_postfix.setter
    def license_key_postfix(self, license_key_postfix):
        """Sets the license_key_postfix of this CommonModelsSCXAccountSCXSubscription.


        :param license_key_postfix: The license_key_postfix of this CommonModelsSCXAccountSCXSubscription.
        :type: str
        """

        self._license_key_postfix = license_key_postfix

    @property
    def subscription_id(self):
        # type: () -> int
        """Gets the subscription_id of this CommonModelsSCXAccountSCXSubscription.


        :return: The subscription_id of this CommonModelsSCXAccountSCXSubscription.
        :rtype: int
        """
        return self._subscription_id

    @subscription_id.setter
    def subscription_id(self, subscription_id):
        """Sets the subscription_id of this CommonModelsSCXAccountSCXSubscription.


        :param subscription_id: The subscription_id of this CommonModelsSCXAccountSCXSubscription.
        :type: int
        """

        self._subscription_id = subscription_id

    @property
    def overage_support(self):
        # type: () -> bool
        """Gets the overage_support of this CommonModelsSCXAccountSCXSubscription.


        :return: The overage_support of this CommonModelsSCXAccountSCXSubscription.
        :rtype: bool
        """
        return self._overage_support

    @overage_support.setter
    def overage_support(self, overage_support):
        """Sets the overage_support of this CommonModelsSCXAccountSCXSubscription.


        :param overage_support: The overage_support of this CommonModelsSCXAccountSCXSubscription.
        :type: bool
        """

        self._overage_support = overage_support

    @property
    def expiration_date(self):
        # type: () -> datetime
        """Gets the expiration_date of this CommonModelsSCXAccountSCXSubscription.


        :return: The expiration_date of this CommonModelsSCXAccountSCXSubscription.
        :rtype: datetime
        """
        return self._expiration_date

    @expiration_date.setter
    def expiration_date(self, expiration_date):
        """Sets the expiration_date of this CommonModelsSCXAccountSCXSubscription.


        :param expiration_date: The expiration_date of this CommonModelsSCXAccountSCXSubscription.
        :type: datetime
        """

        self._expiration_date = expiration_date

    @property
    def renewal_date(self):
        # type: () -> datetime
        """Gets the renewal_date of this CommonModelsSCXAccountSCXSubscription.


        :return: The renewal_date of this CommonModelsSCXAccountSCXSubscription.
        :rtype: datetime
        """
        return self._renewal_date

    @renewal_date.setter
    def renewal_date(self, renewal_date):
        """Sets the renewal_date of this CommonModelsSCXAccountSCXSubscription.


        :param renewal_date: The renewal_date of this CommonModelsSCXAccountSCXSubscription.
        :type: datetime
        """

        self._renewal_date = renewal_date

    @property
    def purchase_date(self):
        # type: () -> datetime
        """Gets the purchase_date of this CommonModelsSCXAccountSCXSubscription.


        :return: The purchase_date of this CommonModelsSCXAccountSCXSubscription.
        :rtype: datetime
        """
        return self._purchase_date

    @purchase_date.setter
    def purchase_date(self, purchase_date):
        """Sets the purchase_date of this CommonModelsSCXAccountSCXSubscription.


        :param purchase_date: The purchase_date of this CommonModelsSCXAccountSCXSubscription.
        :type: datetime
        """

        self._purchase_date = purchase_date

    @property
    def offering_type(self):
        # type: () -> str
        """Gets the offering_type of this CommonModelsSCXAccountSCXSubscription.


        :return: The offering_type of this CommonModelsSCXAccountSCXSubscription.
        :rtype: str
        """
        return self._offering_type

    @offering_type.setter
    def offering_type(self, offering_type):
        """Sets the offering_type of this CommonModelsSCXAccountSCXSubscription.


        :param offering_type: The offering_type of this CommonModelsSCXAccountSCXSubscription.
        :type: str
        """
        allowed_values = [
            "None",
            "Trial",
            "HTrial",
            "Applications",
            "ScanExecution",
            "AnalyzerConcurrent",
            "ConsultantServices",
            "Premium",
            "OpenSourcePerApplication",
            "OpenSourcePremium",
            "OpenSourceConcurrent",
            "IASTConcurrent",
            "IASTPayPerApp",
            "Promotional",
            "Silver",
            "Gold",
            "Platinum",
            "SCAPerApplication",
        ]
        if self._configuration.client_side_validation and offering_type not in allowed_values:
            raise ValueError(
                "Invalid value for `offering_type` ({}), must be one of {}".format(
                    offering_type, allowed_values
                )
            )

        self._offering_type = offering_type

    @property
    def n_seats(self):
        # type: () -> int
        """Gets the n_seats of this CommonModelsSCXAccountSCXSubscription.


        :return: The n_seats of this CommonModelsSCXAccountSCXSubscription.
        :rtype: int
        """
        return self._n_seats

    @n_seats.setter
    def n_seats(self, n_seats):
        """Sets the n_seats of this CommonModelsSCXAccountSCXSubscription.


        :param n_seats: The n_seats of this CommonModelsSCXAccountSCXSubscription.
        :type: int
        """

        self._n_seats = n_seats

    @property
    def n_taken_seats(self):
        # type: () -> int
        """Gets the n_taken_seats of this CommonModelsSCXAccountSCXSubscription.


        :return: The n_taken_seats of this CommonModelsSCXAccountSCXSubscription.
        :rtype: int
        """
        return self._n_taken_seats

    @n_taken_seats.setter
    def n_taken_seats(self, n_taken_seats):
        """Sets the n_taken_seats of this CommonModelsSCXAccountSCXSubscription.


        :param n_taken_seats: The n_taken_seats of this CommonModelsSCXAccountSCXSubscription.
        :type: int
        """

        self._n_taken_seats = n_taken_seats

    @property
    def n_executions(self):
        # type: () -> int
        """Gets the n_executions of this CommonModelsSCXAccountSCXSubscription.


        :return: The n_executions of this CommonModelsSCXAccountSCXSubscription.
        :rtype: int
        """
        return self._n_executions

    @n_executions.setter
    def n_executions(self, n_executions):
        """Sets the n_executions of this CommonModelsSCXAccountSCXSubscription.


        :param n_executions: The n_executions of this CommonModelsSCXAccountSCXSubscription.
        :type: int
        """

        self._n_executions = n_executions

    @property
    def n_apps(self):
        # type: () -> int
        """Gets the n_apps of this CommonModelsSCXAccountSCXSubscription.


        :return: The n_apps of this CommonModelsSCXAccountSCXSubscription.
        :rtype: int
        """
        return self._n_apps

    @n_apps.setter
    def n_apps(self, n_apps):
        """Sets the n_apps of this CommonModelsSCXAccountSCXSubscription.


        :param n_apps: The n_apps of this CommonModelsSCXAccountSCXSubscription.
        :type: int
        """

        self._n_apps = n_apps

    @property
    def full_report(self):
        # type: () -> bool
        """Gets the full_report of this CommonModelsSCXAccountSCXSubscription.


        :return: The full_report of this CommonModelsSCXAccountSCXSubscription.
        :rtype: bool
        """
        return self._full_report

    @full_report.setter
    def full_report(self, full_report):
        """Sets the full_report of this CommonModelsSCXAccountSCXSubscription.


        :param full_report: The full_report of this CommonModelsSCXAccountSCXSubscription.
        :type: bool
        """

        self._full_report = full_report

    @property
    def max_concurrent_scans(self):
        # type: () -> int
        """Gets the max_concurrent_scans of this CommonModelsSCXAccountSCXSubscription.


        :return: The max_concurrent_scans of this CommonModelsSCXAccountSCXSubscription.
        :rtype: int
        """
        return self._max_concurrent_scans

    @max_concurrent_scans.setter
    def max_concurrent_scans(self, max_concurrent_scans):
        """Sets the max_concurrent_scans of this CommonModelsSCXAccountSCXSubscription.


        :param max_concurrent_scans: The max_concurrent_scans of this CommonModelsSCXAccountSCXSubscription.
        :type: int
        """

        self._max_concurrent_scans = max_concurrent_scans

    @property
    def purchase_supplier(self):
        # type: () -> str
        """Gets the purchase_supplier of this CommonModelsSCXAccountSCXSubscription.


        :return: The purchase_supplier of this CommonModelsSCXAccountSCXSubscription.
        :rtype: str
        """
        return self._purchase_supplier

    @purchase_supplier.setter
    def purchase_supplier(self, purchase_supplier):
        """Sets the purchase_supplier of this CommonModelsSCXAccountSCXSubscription.


        :param purchase_supplier: The purchase_supplier of this CommonModelsSCXAccountSCXSubscription.
        :type: str
        """

        self._purchase_supplier = purchase_supplier

    @property
    def state(self):
        # type: () -> str
        """Gets the state of this CommonModelsSCXAccountSCXSubscription.


        :return: The state of this CommonModelsSCXAccountSCXSubscription.
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this CommonModelsSCXAccountSCXSubscription.


        :param state: The state of this CommonModelsSCXAccountSCXSubscription.
        :type: str
        """

        self._state = state

    @property
    def include_open_source(self):
        # type: () -> bool
        """Gets the include_open_source of this CommonModelsSCXAccountSCXSubscription.


        :return: The include_open_source of this CommonModelsSCXAccountSCXSubscription.
        :rtype: bool
        """
        return self._include_open_source

    @include_open_source.setter
    def include_open_source(self, include_open_source):
        """Sets the include_open_source of this CommonModelsSCXAccountSCXSubscription.


        :param include_open_source: The include_open_source of this CommonModelsSCXAccountSCXSubscription.
        :type: bool
        """

        self._include_open_source = include_open_source

    @property
    def include_iast(self):
        # type: () -> bool
        """Gets the include_iast of this CommonModelsSCXAccountSCXSubscription.


        :return: The include_iast of this CommonModelsSCXAccountSCXSubscription.
        :rtype: bool
        """
        return self._include_iast

    @include_iast.setter
    def include_iast(self, include_iast):
        """Sets the include_iast of this CommonModelsSCXAccountSCXSubscription.


        :param include_iast: The include_iast of this CommonModelsSCXAccountSCXSubscription.
        :type: bool
        """

        self._include_iast = include_iast

    @property
    def max_iast_concurrency(self):
        # type: () -> int
        """Gets the max_iast_concurrency of this CommonModelsSCXAccountSCXSubscription.


        :return: The max_iast_concurrency of this CommonModelsSCXAccountSCXSubscription.
        :rtype: int
        """
        return self._max_iast_concurrency

    @max_iast_concurrency.setter
    def max_iast_concurrency(self, max_iast_concurrency):
        """Sets the max_iast_concurrency of this CommonModelsSCXAccountSCXSubscription.


        :param max_iast_concurrency: The max_iast_concurrency of this CommonModelsSCXAccountSCXSubscription.
        :type: int
        """

        self._max_iast_concurrency = max_iast_concurrency

    @property
    def is3_pm_order(self):
        # type: () -> bool
        """Gets the is3_pm_order of this CommonModelsSCXAccountSCXSubscription.


        :return: The is3_pm_order of this CommonModelsSCXAccountSCXSubscription.
        :rtype: bool
        """
        return self._is3_pm_order

    @is3_pm_order.setter
    def is3_pm_order(self, is3_pm_order):
        """Sets the is3_pm_order of this CommonModelsSCXAccountSCXSubscription.


        :param is3_pm_order: The is3_pm_order of this CommonModelsSCXAccountSCXSubscription.
        :type: bool
        """

        self._is3_pm_order = is3_pm_order

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.swagger_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(CommonModelsSCXAccountSCXSubscription, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CommonModelsSCXAccountSCXSubscription):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CommonModelsSCXAccountSCXSubscription):
            return True

        return self.to_dict() != other.to_dict()



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

from pyasoc.configuration import Configuration


class MicrosoftDataEdmIEdmModel:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "schema_elements": "list[MicrosoftDataEdmIEdmSchemaElement]",
        "vocabulary_annotations": "list[MicrosoftDataEdmAnnotationsIEdmVocabularyAnnotation]",
        "referenced_models": "list[MicrosoftDataEdmIEdmModel]",
        "direct_value_annotations_manager": "MicrosoftDataEdmAnnotationsIEdmDirectValueAnnotationsManager",
    }

    attribute_map = {
        "schema_elements": "SchemaElements",
        "vocabulary_annotations": "VocabularyAnnotations",
        "referenced_models": "ReferencedModels",
        "direct_value_annotations_manager": "DirectValueAnnotationsManager",
    }

    def __init__(
        self,
        schema_elements=None,
        vocabulary_annotations=None,
        referenced_models=None,
        direct_value_annotations_manager=None,
        _configuration=None,
    ):
        """MicrosoftDataEdmIEdmModel - a model defined in Swagger"""
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._schema_elements = None
        self._vocabulary_annotations = None
        self._referenced_models = None
        self._direct_value_annotations_manager = None
        self.discriminator = None

        if schema_elements is not None:
            self.schema_elements = schema_elements
        if vocabulary_annotations is not None:
            self.vocabulary_annotations = vocabulary_annotations
        if referenced_models is not None:
            self.referenced_models = referenced_models
        if direct_value_annotations_manager is not None:
            self.direct_value_annotations_manager = direct_value_annotations_manager

    @property
    def schema_elements(self):
        # type: () -> list[MicrosoftDataEdmIEdmSchemaElement]
        """Gets the schema_elements of this MicrosoftDataEdmIEdmModel.


        :return: The schema_elements of this MicrosoftDataEdmIEdmModel.
        :rtype: list[MicrosoftDataEdmIEdmSchemaElement]
        """
        return self._schema_elements

    @schema_elements.setter
    def schema_elements(self, schema_elements):
        """Sets the schema_elements of this MicrosoftDataEdmIEdmModel.


        :param schema_elements: The schema_elements of this MicrosoftDataEdmIEdmModel.
        :type: list[MicrosoftDataEdmIEdmSchemaElement]
        """

        self._schema_elements = schema_elements

    @property
    def vocabulary_annotations(self):
        # type: () -> list[MicrosoftDataEdmAnnotationsIEdmVocabularyAnnotation]
        """Gets the vocabulary_annotations of this MicrosoftDataEdmIEdmModel.


        :return: The vocabulary_annotations of this MicrosoftDataEdmIEdmModel.
        :rtype: list[MicrosoftDataEdmAnnotationsIEdmVocabularyAnnotation]
        """
        return self._vocabulary_annotations

    @vocabulary_annotations.setter
    def vocabulary_annotations(self, vocabulary_annotations):
        """Sets the vocabulary_annotations of this MicrosoftDataEdmIEdmModel.


        :param vocabulary_annotations: The vocabulary_annotations of this MicrosoftDataEdmIEdmModel.
        :type: list[MicrosoftDataEdmAnnotationsIEdmVocabularyAnnotation]
        """

        self._vocabulary_annotations = vocabulary_annotations

    @property
    def referenced_models(self):
        # type: () -> list[MicrosoftDataEdmIEdmModel]
        """Gets the referenced_models of this MicrosoftDataEdmIEdmModel.


        :return: The referenced_models of this MicrosoftDataEdmIEdmModel.
        :rtype: list[MicrosoftDataEdmIEdmModel]
        """
        return self._referenced_models

    @referenced_models.setter
    def referenced_models(self, referenced_models):
        """Sets the referenced_models of this MicrosoftDataEdmIEdmModel.


        :param referenced_models: The referenced_models of this MicrosoftDataEdmIEdmModel.
        :type: list[MicrosoftDataEdmIEdmModel]
        """

        self._referenced_models = referenced_models

    @property
    def direct_value_annotations_manager(self):
        # type: () -> MicrosoftDataEdmAnnotationsIEdmDirectValueAnnotationsManager
        """Gets the direct_value_annotations_manager of this MicrosoftDataEdmIEdmModel.


        :return: The direct_value_annotations_manager of this MicrosoftDataEdmIEdmModel.
        :rtype: MicrosoftDataEdmAnnotationsIEdmDirectValueAnnotationsManager
        """
        return self._direct_value_annotations_manager

    @direct_value_annotations_manager.setter
    def direct_value_annotations_manager(self, direct_value_annotations_manager):
        """Sets the direct_value_annotations_manager of this MicrosoftDataEdmIEdmModel.


        :param direct_value_annotations_manager: The direct_value_annotations_manager of this MicrosoftDataEdmIEdmModel.
        :type: MicrosoftDataEdmAnnotationsIEdmDirectValueAnnotationsManager
        """

        self._direct_value_annotations_manager = direct_value_annotations_manager

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.swagger_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(MicrosoftDataEdmIEdmModel, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MicrosoftDataEdmIEdmModel):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MicrosoftDataEdmIEdmModel):
            return True

        return self.to_dict() != other.to_dict()



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

from pyasoc.configuration import Configuration


class CommonModelsDomainModel:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "id": "int",
        "domain": "str",
        "enabled": "bool",
        "type": "str",
        "status": "str",
        "key": "str",
        "added_by": "str",
        "created_at": "datetime",
    }

    attribute_map = {
        "id": "Id",
        "domain": "Domain",
        "enabled": "Enabled",
        "type": "Type",
        "status": "Status",
        "key": "Key",
        "added_by": "AddedBy",
        "created_at": "CreatedAt",
    }

    def __init__(
        self,
        id=None,
        domain=None,
        enabled=None,
        type=None,
        status=None,
        key=None,
        added_by=None,
        created_at=None,
        _configuration=None,
    ):
        """CommonModelsDomainModel - a model defined in Swagger"""
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._id = None
        self._domain = None
        self._enabled = None
        self._type = None
        self._status = None
        self._key = None
        self._added_by = None
        self._created_at = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if domain is not None:
            self.domain = domain
        if enabled is not None:
            self.enabled = enabled
        if type is not None:
            self.type = type
        if status is not None:
            self.status = status
        if key is not None:
            self.key = key
        if added_by is not None:
            self.added_by = added_by
        if created_at is not None:
            self.created_at = created_at

    @property
    def id(self):
        # type: () -> int
        """Gets the id of this CommonModelsDomainModel.


        :return: The id of this CommonModelsDomainModel.
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this CommonModelsDomainModel.


        :param id: The id of this CommonModelsDomainModel.
        :type: int
        """

        self._id = id

    @property
    def domain(self):
        # type: () -> str
        """Gets the domain of this CommonModelsDomainModel.


        :return: The domain of this CommonModelsDomainModel.
        :rtype: str
        """
        return self._domain

    @domain.setter
    def domain(self, domain):
        """Sets the domain of this CommonModelsDomainModel.


        :param domain: The domain of this CommonModelsDomainModel.
        :type: str
        """

        self._domain = domain

    @property
    def enabled(self):
        # type: () -> bool
        """Gets the enabled of this CommonModelsDomainModel.


        :return: The enabled of this CommonModelsDomainModel.
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this CommonModelsDomainModel.


        :param enabled: The enabled of this CommonModelsDomainModel.
        :type: bool
        """

        self._enabled = enabled

    @property
    def type(self):
        # type: () -> str
        """Gets the type of this CommonModelsDomainModel.


        :return: The type of this CommonModelsDomainModel.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this CommonModelsDomainModel.


        :param type: The type of this CommonModelsDomainModel.
        :type: str
        """
        allowed_values = ["Manually", "Html", "Email", "DnsComparison"]
        if self._configuration.client_side_validation and type not in allowed_values:
            raise ValueError(f"Invalid value for `type` ({type}), must be one of {allowed_values}")

        self._type = type

    @property
    def status(self):
        # type: () -> str
        """Gets the status of this CommonModelsDomainModel.


        :return: The status of this CommonModelsDomainModel.
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this CommonModelsDomainModel.


        :param status: The status of this CommonModelsDomainModel.
        :type: str
        """
        allowed_values = ["None", "Verified", "Pending"]
        if self._configuration.client_side_validation and status not in allowed_values:
            raise ValueError(
                f"Invalid value for `status` ({status}), must be one of {allowed_values}"
            )

        self._status = status

    @property
    def key(self):
        # type: () -> str
        """Gets the key of this CommonModelsDomainModel.


        :return: The key of this CommonModelsDomainModel.
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this CommonModelsDomainModel.


        :param key: The key of this CommonModelsDomainModel.
        :type: str
        """

        self._key = key

    @property
    def added_by(self):
        # type: () -> str
        """Gets the added_by of this CommonModelsDomainModel.


        :return: The added_by of this CommonModelsDomainModel.
        :rtype: str
        """
        return self._added_by

    @added_by.setter
    def added_by(self, added_by):
        """Sets the added_by of this CommonModelsDomainModel.


        :param added_by: The added_by of this CommonModelsDomainModel.
        :type: str
        """

        self._added_by = added_by

    @property
    def created_at(self):
        # type: () -> datetime
        """Gets the created_at of this CommonModelsDomainModel.


        :return: The created_at of this CommonModelsDomainModel.
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this CommonModelsDomainModel.


        :param created_at: The created_at of this CommonModelsDomainModel.
        :type: datetime
        """

        self._created_at = created_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.swagger_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(CommonModelsDomainModel, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CommonModelsDomainModel):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CommonModelsDomainModel):
            return True

        return self.to_dict() != other.to_dict()



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import re  # noqa: F401

# python 2 and python 3 compatibility library
from pyasoc.api_client import ApiClient


class ToolsApi:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def tools_download(self, **kwargs):
        """Download IAST Agent

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.tools_download(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str type: Agent type (Java or DotNet)
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.tools_download_with_http_info(**kwargs)
        else:
            (data) = self.tools_download_with_http_info(**kwargs)
            return data

    def tools_download_with_http_info(self, **kwargs):
        """Download IAST Agent

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.tools_download_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str type: Agent type (Java or DotNet)
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["type"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'" " to method tools_download" % key
                )
            params[key] = val
        del params["kwargs"]

        collection_formats = {}

        path_params = {}

        query_params = []
        if "type" in params:
            query_params.append(("type", params["type"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/octet-stream"])

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Tools/IAST/Download",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="file",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def tools_download_with_key(self, scan_id, **kwargs):
        """Download IAST agent with access key.

        Note: This operation invalidates the previous key.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.tools_download_with_key(scan_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str scan_id: Scan identifier (required)
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.tools_download_with_key_with_http_info(scan_id, **kwargs)
        else:
            (data) = self.tools_download_with_key_with_http_info(scan_id, **kwargs)
            return data

    def tools_download_with_key_with_http_info(self, scan_id, **kwargs):
        """Download IAST agent with access key.

        Note: This operation invalidates the previous key.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.tools_download_with_key_with_http_info(scan_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str scan_id: Scan identifier (required)
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["scan_id"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method tools_download_with_key" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'scan_id' is set
        if self.api_client.client_side_validation and (
            "scan_id" not in params or params["scan_id"] is None
        ):
            raise ValueError(
                "Missing the required parameter `scan_id` when calling `tools_download_with_key`"
            )

        collection_formats = {}

        path_params = {}

        query_params = []
        if "scan_id" in params:
            query_params.append(("scanId", params["scan_id"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/octet-stream"])

        # Authentication setting
        auth_settings = ["AccessToken"]

        return self.api_client.call_api(
            "/api/V2/Tools/IAST/DownloadWithKey",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="file",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def tools_presence_v2_by_platform(self, platform, **kwargs):
        """Download Presence V2 package

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.tools_presence_v2_by_platform(platform, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str platform: Platform - osx_x64 is not yet supported !! (required)
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.tools_presence_v2_by_platform_with_http_info(platform, **kwargs)
        else:
            (data) = self.tools_presence_v2_by_platform_with_http_info(platform, **kwargs)
            return data

    def tools_presence_v2_by_platform_with_http_info(self, platform, **kwargs):
        """Download Presence V2 package

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.tools_presence_v2_by_platform_with_http_info(platform, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str platform: Platform - osx_x64 is not yet supported !! (required)
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["platform"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method tools_presence_v2_by_platform" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'platform' is set
        if self.api_client.client_side_validation and (
            "platform" not in params or params["platform"] is None
        ):
            raise ValueError(
                "Missing the required parameter `platform` when calling `tools_presence_v2_by_platform`"
            )

        collection_formats = {}

        path_params = {}
        if "platform" in params:
            path_params["platform"] = params["platform"]

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/octet-stream"])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            "/api/V2/Tools/PresenceV2/{platform}",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="file",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )

    def tools_traffic_recorder_by_platform(self, platform, **kwargs):
        """Download a Standalone Traffic Recorder - used for recording traffic for dast scans.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.tools_traffic_recorder_by_platform(platform, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str platform: Platform (required)
        :param bool version: If this parameter is provided with value true, only the version id is returned
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs["_return_http_data_only"] = True
        if kwargs.get("async_req"):
            return self.tools_traffic_recorder_by_platform_with_http_info(platform, **kwargs)
        else:
            (data) = self.tools_traffic_recorder_by_platform_with_http_info(platform, **kwargs)
            return data

    def tools_traffic_recorder_by_platform_with_http_info(self, platform, **kwargs):
        """Download a Standalone Traffic Recorder - used for recording traffic for dast scans.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.tools_traffic_recorder_by_platform_with_http_info(platform, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str platform: Platform (required)
        :param bool version: If this parameter is provided with value true, only the version id is returned
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ["platform", "version"]
        all_params.append("async_req")
        all_params.append("_return_http_data_only")
        all_params.append("_preload_content")
        all_params.append("_request_timeout")

        params = locals()
        for key, val in params["kwargs"].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method tools_traffic_recorder_by_platform" % key
                )
            params[key] = val
        del params["kwargs"]
        # verify the required parameter 'platform' is set
        if self.api_client.client_side_validation and (
            "platform" not in params or params["platform"] is None
        ):
            raise ValueError(
                "Missing the required parameter `platform` when calling `tools_traffic_recorder_by_platform`"
            )

        collection_formats = {}

        path_params = {}
        if "platform" in params:
            path_params["platform"] = params["platform"]

        query_params = []
        if "version" in params:
            query_params.append(("version", params["version"]))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params["Accept"] = self.api_client.select_header_accept(["application/octet-stream"])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            "/api/V2/Tools/TrafficRecorder/{platform}",
            "GET",
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type="file",
            auth_settings=auth_settings,
            async_req=params.get("async_req"),
            _return_http_data_only=params.get("_return_http_data_only"),
            _preload_content=params.get("_preload_content", True),
            _request_timeout=params.get("_request_timeout"),
            collection_formats=collection_formats,
        )



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

from pyasoc.configuration import Configuration


class UserSiteCommonModelsNewIastAnalyzerScan:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "conn_lost_stop_timer": "int",
        "agent_type": "str",
        "scan_name": "str",
        "enable_mail_notification": "bool",
        "locale": "str",
        "app_id": "str",
        "execute": "bool",
        "fully_automatic": "bool",
        "personal": "bool",
        "comment": "str",
    }

    attribute_map = {
        "conn_lost_stop_timer": "ConnLostStopTimer",
        "agent_type": "AgentType",
        "scan_name": "ScanName",
        "enable_mail_notification": "EnableMailNotification",
        "locale": "Locale",
        "app_id": "AppId",
        "execute": "Execute",
        "fully_automatic": "FullyAutomatic",
        "personal": "Personal",
        "comment": "Comment",
    }

    def __init__(
        self,
        conn_lost_stop_timer=None,
        agent_type=None,
        scan_name=None,
        enable_mail_notification=None,
        locale=None,
        app_id=None,
        execute=None,
        fully_automatic=None,
        personal=None,
        comment=None,
        _configuration=None,
    ):
        """UserSiteCommonModelsNewIastAnalyzerScan - a model defined in Swagger"""
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._conn_lost_stop_timer = None
        self._agent_type = None
        self._scan_name = None
        self._enable_mail_notification = None
        self._locale = None
        self._app_id = None
        self._execute = None
        self._fully_automatic = None
        self._personal = None
        self._comment = None
        self.discriminator = None

        if conn_lost_stop_timer is not None:
            self.conn_lost_stop_timer = conn_lost_stop_timer
        if agent_type is not None:
            self.agent_type = agent_type
        self.scan_name = scan_name
        if enable_mail_notification is not None:
            self.enable_mail_notification = enable_mail_notification
        if locale is not None:
            self.locale = locale
        self.app_id = app_id
        if execute is not None:
            self.execute = execute
        if fully_automatic is not None:
            self.fully_automatic = fully_automatic
        if personal is not None:
            self.personal = personal
        if comment is not None:
            self.comment = comment

    @property
    def conn_lost_stop_timer(self):
        # type: () -> int
        """Gets the conn_lost_stop_timer of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The conn_lost_stop_timer of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: int
        """
        return self._conn_lost_stop_timer

    @conn_lost_stop_timer.setter
    def conn_lost_stop_timer(self, conn_lost_stop_timer):
        """Sets the conn_lost_stop_timer of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param conn_lost_stop_timer: The conn_lost_stop_timer of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: int
        """
        if (
            self._configuration.client_side_validation
            and conn_lost_stop_timer is not None
            and conn_lost_stop_timer > 2147483647
        ):
            raise ValueError(
                "Invalid value for `conn_lost_stop_timer`, must be a value less than or equal to `2147483647`"
            )
        if (
            self._configuration.client_side_validation
            and conn_lost_stop_timer is not None
            and conn_lost_stop_timer < 30
        ):
            raise ValueError(
                "Invalid value for `conn_lost_stop_timer`, must be a value greater than or equal to `30`"
            )

        self._conn_lost_stop_timer = conn_lost_stop_timer

    @property
    def agent_type(self):
        # type: () -> str
        """Gets the agent_type of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The agent_type of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: str
        """
        return self._agent_type

    @agent_type.setter
    def agent_type(self, agent_type):
        """Sets the agent_type of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param agent_type: The agent_type of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: str
        """
        allowed_values = ["Java", "DotNet", "NodeJS"]
        if self._configuration.client_side_validation and agent_type not in allowed_values:
            raise ValueError(
                "Invalid value for `agent_type` ({}), must be one of {}".format(
                    agent_type, allowed_values
                )
            )

        self._agent_type = agent_type

    @property
    def scan_name(self):
        # type: () -> str
        """Gets the scan_name of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The scan_name of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: str
        """
        return self._scan_name

    @scan_name.setter
    def scan_name(self, scan_name):
        """Sets the scan_name of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param scan_name: The scan_name of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: str
        """
        if self._configuration.client_side_validation and scan_name is None:
            raise ValueError("Invalid value for `scan_name`, must not be `None`")
        if (
            self._configuration.client_side_validation
            and scan_name is not None
            and len(scan_name) > 1024
        ):
            raise ValueError(
                "Invalid value for `scan_name`, length must be less than or equal to `1024`"
            )
        if (
            self._configuration.client_side_validation
            and scan_name is not None
            and len(scan_name) < 1
        ):
            raise ValueError(
                "Invalid value for `scan_name`, length must be greater than or equal to `1`"
            )

        self._scan_name = scan_name

    @property
    def enable_mail_notification(self):
        # type: () -> bool
        """Gets the enable_mail_notification of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The enable_mail_notification of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: bool
        """
        return self._enable_mail_notification

    @enable_mail_notification.setter
    def enable_mail_notification(self, enable_mail_notification):
        """Sets the enable_mail_notification of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param enable_mail_notification: The enable_mail_notification of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: bool
        """

        self._enable_mail_notification = enable_mail_notification

    @property
    def locale(self):
        # type: () -> str
        """Gets the locale of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The locale of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: str
        """
        return self._locale

    @locale.setter
    def locale(self, locale):
        """Sets the locale of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param locale: The locale of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: str
        """
        if self._configuration.client_side_validation and locale is not None and len(locale) > 10:
            raise ValueError(
                "Invalid value for `locale`, length must be less than or equal to `10`"
            )
        if self._configuration.client_side_validation and locale is not None and len(locale) < 0:
            raise ValueError(
                "Invalid value for `locale`, length must be greater than or equal to `0`"
            )

        self._locale = locale

    @property
    def app_id(self):
        # type: () -> str
        """Gets the app_id of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The app_id of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: str
        """
        return self._app_id

    @app_id.setter
    def app_id(self, app_id):
        """Sets the app_id of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param app_id: The app_id of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: str
        """
        if self._configuration.client_side_validation and app_id is None:
            raise ValueError("Invalid value for `app_id`, must not be `None`")

        self._app_id = app_id

    @property
    def execute(self):
        # type: () -> bool
        """Gets the execute of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The execute of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: bool
        """
        return self._execute

    @execute.setter
    def execute(self, execute):
        """Sets the execute of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param execute: The execute of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: bool
        """

        self._execute = execute

    @property
    def fully_automatic(self):
        # type: () -> bool
        """Gets the fully_automatic of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The fully_automatic of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: bool
        """
        return self._fully_automatic

    @fully_automatic.setter
    def fully_automatic(self, fully_automatic):
        """Sets the fully_automatic of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param fully_automatic: The fully_automatic of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: bool
        """

        self._fully_automatic = fully_automatic

    @property
    def personal(self):
        # type: () -> bool
        """Gets the personal of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The personal of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: bool
        """
        return self._personal

    @personal.setter
    def personal(self, personal):
        """Sets the personal of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param personal: The personal of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: bool
        """

        self._personal = personal

    @property
    def comment(self):
        # type: () -> str
        """Gets the comment of this UserSiteCommonModelsNewIastAnalyzerScan.


        :return: The comment of this UserSiteCommonModelsNewIastAnalyzerScan.
        :rtype: str
        """
        return self._comment

    @comment.setter
    def comment(self, comment):
        """Sets the comment of this UserSiteCommonModelsNewIastAnalyzerScan.


        :param comment: The comment of this UserSiteCommonModelsNewIastAnalyzerScan.
        :type: str
        """
        if (
            self._configuration.client_side_validation
            and comment is not None
            and len(comment) > 2048
        ):
            raise ValueError(
                "Invalid value for `comment`, length must be less than or equal to `2048`"
            )
        if self._configuration.client_side_validation and comment is not None and len(comment) < 0:
            raise ValueError(
                "Invalid value for `comment`, length must be greater than or equal to `0`"
            )

        self._comment = comment

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.swagger_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(UserSiteCommonModelsNewIastAnalyzerScan, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UserSiteCommonModelsNewIastAnalyzerScan):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, UserSiteCommonModelsNewIastAnalyzerScan):
            return True

        return self.to_dict() != other.to_dict()



"""
    HCL AppScan on Cloud (v2)

    This API allows you to interact with the service. The API allows you to perform many of the operations available in the UI and more.</br>For authentication, use the relevant APIs in the Account section. A successful authentication response includes a bearer token for use in subsequent API calls. Pasting this token in the 'Access token' field above will automatically add the authorization header to any API call that requires a valid session.

    OpenAPI spec version: v2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

from pyasoc.configuration import Configuration


class CommonModelsASMModelsComplianceStatus:
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "policy_id": "str",
        "enabled": "bool",
        "name": "str",
        "associted_by": "str",
        "compliant": "bool",
        "category": "str",
        "parameters": "str",
    }

    attribute_map = {
        "policy_id": "PolicyId",
        "enabled": "Enabled",
        "name": "Name",
        "associted_by": "AssocitedBy",
        "compliant": "Compliant",
        "category": "Category",
        "parameters": "Parameters",
    }

    def __init__(
        self,
        policy_id=None,
        enabled=None,
        name=None,
        associted_by=None,
        compliant=None,
        category=None,
        parameters=None,
        _configuration=None,
    ):
        """CommonModelsASMModelsComplianceStatus - a model defined in Swagger"""
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._policy_id = None
        self._enabled = None
        self._name = None
        self._associted_by = None
        self._compliant = None
        self._category = None
        self._parameters = None
        self.discriminator = None

        if policy_id is not None:
            self.policy_id = policy_id
        if enabled is not None:
            self.enabled = enabled
        if name is not None:
            self.name = name
        if associted_by is not None:
            self.associted_by = associted_by
        if compliant is not None:
            self.compliant = compliant
        if category is not None:
            self.category = category
        if parameters is not None:
            self.parameters = parameters

    @property
    def policy_id(self):
        # type: () -> str
        """Gets the policy_id of this CommonModelsASMModelsComplianceStatus.


        :return: The policy_id of this CommonModelsASMModelsComplianceStatus.
        :rtype: str
        """
        return self._policy_id

    @policy_id.setter
    def policy_id(self, policy_id):
        """Sets the policy_id of this CommonModelsASMModelsComplianceStatus.


        :param policy_id: The policy_id of this CommonModelsASMModelsComplianceStatus.
        :type: str
        """

        self._policy_id = policy_id

    @property
    def enabled(self):
        # type: () -> bool
        """Gets the enabled of this CommonModelsASMModelsComplianceStatus.


        :return: The enabled of this CommonModelsASMModelsComplianceStatus.
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this CommonModelsASMModelsComplianceStatus.


        :param enabled: The enabled of this CommonModelsASMModelsComplianceStatus.
        :type: bool
        """

        self._enabled = enabled

    @property
    def name(self):
        # type: () -> str
        """Gets the name of this CommonModelsASMModelsComplianceStatus.


        :return: The name of this CommonModelsASMModelsComplianceStatus.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this CommonModelsASMModelsComplianceStatus.


        :param name: The name of this CommonModelsASMModelsComplianceStatus.
        :type: str
        """
        if self._configuration.client_side_validation and name is not None and len(name) > 128:
            raise ValueError("Invalid value for `name`, length must be less than or equal to `128`")
        if self._configuration.client_side_validation and name is not None and len(name) < 0:
            raise ValueError(
                "Invalid value for `name`, length must be greater than or equal to `0`"
            )

        self._name = name

    @property
    def associted_by(self):
        # type: () -> str
        """Gets the associted_by of this CommonModelsASMModelsComplianceStatus.


        :return: The associted_by of this CommonModelsASMModelsComplianceStatus.
        :rtype: str
        """
        return self._associted_by

    @associted_by.setter
    def associted_by(self, associted_by):
        """Sets the associted_by of this CommonModelsASMModelsComplianceStatus.


        :param associted_by: The associted_by of this CommonModelsASMModelsComplianceStatus.
        :type: str
        """

        self._associted_by = associted_by

    @property
    def compliant(self):
        # type: () -> bool
        """Gets the compliant of this CommonModelsASMModelsComplianceStatus.


        :return: The compliant of this CommonModelsASMModelsComplianceStatus.
        :rtype: bool
        """
        return self._compliant

    @compliant.setter
    def compliant(self, compliant):
        """Sets the compliant of this CommonModelsASMModelsComplianceStatus.


        :param compliant: The compliant of this CommonModelsASMModelsComplianceStatus.
        :type: bool
        """

        self._compliant = compliant

    @property
    def category(self):
        # type: () -> str
        """Gets the category of this CommonModelsASMModelsComplianceStatus.


        :return: The category of this CommonModelsASMModelsComplianceStatus.
        :rtype: str
        """
        return self._category

    @category.setter
    def category(self, category):
        """Sets the category of this CommonModelsASMModelsComplianceStatus.


        :param category: The category of this CommonModelsASMModelsComplianceStatus.
        :type: str
        """
        allowed_values = [
            "Custom",
            "Security",
            "Regulation",
            "IndustryStandard",
        ]
        if self._configuration.client_side_validation and category not in allowed_values:
            raise ValueError(
                "Invalid value for `category` ({}), must be one of {}".format(
                    category, allowed_values
                )
            )

        self._category = category

    @property
    def parameters(self):
        # type: () -> str
        """Gets the parameters of this CommonModelsASMModelsComplianceStatus.


        :return: The parameters of this CommonModelsASMModelsComplianceStatus.
        :rtype: str
        """
        return self._parameters

    @parameters.setter
    def parameters(self, parameters):
        """Sets the parameters of this CommonModelsASMModelsComplianceStatus.


        :param parameters: The parameters of this CommonModelsASMModelsComplianceStatus.
        :type: str
        """

        self._parameters = parameters

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in self.swagger_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(CommonModelsASMModelsComplianceStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CommonModelsASMModelsComplianceStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CommonModelsASMModelsComplianceStatus):
            return True

        return self.to_dict() != other.to_dict()
